@@ -343,6 +343,14 @@ val db_close : db -> bool
     @raise SqliteError if an invalid database handle is passed.
 *)
 
+val ( let& ) : db -> (db -> 'a) -> 'a
+(** [let& db = db_open "..." in ...scope that uses db...] ensures that the
+    database [db] is safely closed at the end of the scope, even if there
+    is an exception somewhere in the scope.
+
+    @raise Fun.Finally_raised if the database could not be closed successfully.
+*)
+
 val enable_load_extension : db -> bool -> bool
 (** [enable_load_extension db onoff] enable/disable the SQLite3 load
     extension.  @return [false] if the operation fails, [true]
@@ -373,9 +381,9 @@ val exec : db -> ?cb : (row -> headers -> unit) -> string -> Rc.t
 (** [exec db ?cb sql] performs SQL-operation [sql] on database [db].
     If the operation contains query statements, then the callback function
     [cb] will be called for each matching row.  The first parameter of
-    the callback is the contents of the row, the second paramater are the
-    headers of the columns associated with the row.  Exceptions raised
-    within the callback will abort the execution and escape {!exec}.
+    the callback contains the contents of the row, the second parameter
+    contains the headers of the columns associated with the row.  Exceptions
+    raised within the callback will abort the execution and escape {!exec}.
 
     @return the return code of the operation.
 
